precision highp float;
precision highp int;

uniform sampler2D u_DiffuseTexture;

uniform vec3 u_E_LightDirection;
uniform float u_AmbientFactor;

uniform vec3 u_LightDiffuse;
uniform vec3 u_LightSpecular;

uniform float u_SpecularIntensity;
uniform float u_SpecularHardness;
uniform float u_SpecularTint;
uniform float u_Saturation;

uniform int u_alphaMode; //0: no alpha, 1: alpha = alpha chan, 2: specular = alpha chan

varying vec3 ex_E_Position;
varying vec3 ex_E_Normal;
varying vec2 ex_TexCoord;

//const float _LitOutlineThickness = 1.0 / 128.0;
//const float _UnlitOutlineThickness = 1.0 / 128.0;
//const bool outline = true;

vec3 desaturate(vec3 color, float amount)
{
	vec3 gray = vec3(dot(vec3(0.2126,0.7152,0.0722), color));
	return vec3(mix(color, gray, amount));
}

void main()
{
	vec3 Diffuse = texture2D( u_DiffuseTexture, ex_TexCoord ).rgb;
	//Diffuse = vec3(0.7, 0.1, 0.1);

	vec3 E_LightVec = normalize(-u_E_LightDirection);

	vec3 E_Normal = normalize(ex_E_Normal);
	float LambertFactor = max( dot( E_Normal, E_LightVec ), 0.0 ) + u_AmbientFactor;

	vec4 Final = vec4(0.0, 0.0, 0.0, 1.0);
	if( LambertFactor > 0.0 )
	{
		float ShadowFactor = 1.0;

		vec3 E_EyeDir = normalize( -ex_E_Position );
		vec3 E_HalfVector = normalize( E_LightVec + E_EyeDir );

		float SpecularFactor = max( dot( E_Normal, E_HalfVector ), 0.0 );
		float Specular;
		if (u_alphaMode == 2) {
			Specular = pow( SpecularFactor, u_SpecularHardness ) * u_SpecularIntensity * texture2D( u_DiffuseTexture, ex_TexCoord ).a;
		}
		else {
			Specular = pow( SpecularFactor, u_SpecularHardness ) * u_SpecularIntensity;
		}

		vec3 D = Diffuse * LambertFactor * u_LightDiffuse;
		vec3 SLight = mix(u_LightSpecular, Diffuse * u_LightSpecular, u_SpecularTint);
		vec3 S = Specular * SLight;

		if (u_alphaMode == 0) {
			Final = vec4(desaturate(S + D, 1.0 - u_Saturation), 1.0);
		}
		else if (u_alphaMode == 2) {
			Final = vec4(desaturate(S + D, 1.0 - u_Saturation), 1.0);
		}
		else if (u_alphaMode == 1) {
			Final = vec4(desaturate(S + D, 1.0 - u_Saturation), texture2D( u_DiffuseTexture, ex_TexCoord ).a);
		}
	}
	gl_FragColor = Final;
}
